@@ -1,5 +1,5 @@
 # Device::Modem - a Perl class to interface generic modems (AT-compliant)
-# Copyright (C) 2002-2006 Cosimo Streppone, cosimo@cpan.org
+# Copyright (C) 2002-2010 Cosimo Streppone, cosimo@cpan.org
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
@@ -9,13 +9,10 @@
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # Perl licensing terms for details.
 #
-# Commercial support is available. Write me if you are
-# interested in new features or software support.
-#
-# $Id: Modem.pm,v 1.47 2006/04/20 20:24:05 cosimo Exp $
+# $Id$
 
 package Device::Modem;
-$VERSION = sprintf '%d.%02d', q$Revision: 1.47 $ =~ /(\d)\.(\d+)/;
+$VERSION = '1.52';
 
 BEGIN {
 
@@ -45,11 +42,10 @@ BEGIN {
 }
 
 use strict;
+use Carp ();
 
 # Constants definition
 use constant CTRL_Z => chr(26);
-
-# TODO: Allow to redefine CR to "\r", "\n" or "\r\n"
 use constant CR => "\r";
 
 # Connection defaults
@@ -59,9 +55,9 @@ $Device::Modem::DATABITS = 8;
 $Device::Modem::STOPBITS = 1;
 $Device::Modem::PARITY   = 'none';
 $Device::Modem::TIMEOUT  = 500;     # milliseconds
-$Device::Modem::WAITCYCLE= 20;
+$Device::Modem::WAITCYCLE= 50;
 $Device::Modem::READCHARS= 130;
-$Device::Modem::WAITCMD  = 20; #100;     # milliseconds
+$Device::Modem::WAITCMD  = 200;     # milliseconds
 
 # Setup text and numerical response codes
 @Device::Modem::RESPONSE = ( 'OK', undef, 'RING', 'NO CARRIER', 'ERROR', undef, 'NO DIALTONE', 'BUSY' );
@@ -290,7 +286,7 @@ sub S_register {
             $self->log->write('info', 'value of S'.$register.' register seems to be ['.$value.']');
         } else {
             $value = undef;
-            $self->log->write('error', 'error reading value of S'.$register.' register');
+            $self->log->write('err', 'error reading value of S'.$register.' register');
         }
 
     }
@@ -479,7 +475,7 @@ sub connect {
 
     # Check connection
     unless( ref $me->port ) {
-        $me->log->write( 'error', '*FAILED* connect on '.$me->{'port'} );
+        $me->log->write( 'err', '*FAILED* connect on '.$me->{'port'} );
         return $lOk;
     }
 
@@ -507,7 +503,7 @@ sub connect {
     $oPort -> lookclear;
 
     unless ( $oPort -> write_settings ) {
-        $me->log->write( 'error', '*FAILED* write_settings on '.$me->{'port'} );
+        $me->log->write('err', '*FAILED* write_settings on '.$me->{'port'} );
         return $lOk;
     }
     $oPort -> purge_all;
@@ -543,9 +539,39 @@ sub options {
 
 # returns Device::SerialPort object handle
 sub port {
-    my $self = shift();
-    @_ ? $self->{'_comm_object'} = shift()
-      : $self->{'_comm_object'};
+    my $self = shift;
+
+    if (@_) {
+        return ($self->{'_comm_object'} = shift);
+    }
+
+    my $port_obj = $self->{'_comm_object'};
+
+    # Maybe the port was disconnected?
+    if ($self->{'CONNECTED'} == 1 &&                # We were connected
+        (! defined $port_obj || ! $port_obj)) {     # Now we aren't anymore
+
+        # Avoid recursion on ourselves
+        $self->{'CONNECTED'} = 0;
+
+        # Try to reconnect if possible
+        my $connect_options = $self->options;
+
+        # No connect options probably because we didn't ever connect
+        if (! $connect_options) {
+            Carp::croak("Not connected");
+        }
+
+        $self->connect(%{ $connect_options });
+        $port_obj = $self->{'_comm_object'};
+    }
+
+    # Still not connected? bail out
+    if (! defined $port_obj || ! $port_obj) {
+        Carp::croak("Not connected");
+    }
+
+    return $port_obj;
 }
 
 # disconnect serial port
@@ -563,15 +589,39 @@ sub atsend {
     # Write message on port
     $me->port->purge_all();
     $cnt = $me->port->write($msg);
-#    $me->wait($Device::Modem::WAITCMD);
 
-    $me->port->write_drain() unless $me->ostype eq 'windoze';
+    my $lbuf=length($msg);
+    my $ret;
+
+    while ($cnt < $lbuf)
+    {
+       $ret = $me->port->write(substr($msg, $cnt));
+       $me->write_drain();
+       last unless defined $ret;
+       $cnt += $ret;
+    }
+
     $me->log->write('debug', 'atsend: wrote '.$cnt.'/'.length($msg).' chars');
 
     # If wrote all chars of `msg', we are successful
     return $cnt == length $msg;
 }
 
+# Call write_drain() if platform allows to (no call for Win32)
+sub write_drain
+{
+    my $me = shift;
+
+    # No write_drain() call for win32 systems
+    return if $me->ostype eq 'windoze';
+
+    # No write_drain() if no port object available
+    my $port = $me->port;
+    return unless $port;
+
+    return $port->write_drain();
+}
+
 # answer() takes strings from the device until a pattern
 # is encountered or a timeout happens.
 sub _answer {
@@ -894,8 +944,8 @@ modem is matched against this parameter. If input matches, result is returned.
 
 =item *
 
-C<$timeout> - Expressed in seconds. After that time, answer returns result also if nothing
-has been received. Example: C<10>. Default: C<0.2>
+C<$timeout> - Expressed in milliseconds. After that time, answer returns result also if nothing
+has been received. Example: C<10000>. Default: C<$Device::Modem::TIMEOUT>, currently 500 ms.
 
 =back
 
@@ -1178,7 +1228,6 @@ C<$result> would hold "C<OK>" and C<@lines> would consist of only 1 line with
 the string "C<US Robotics 56K Message>".
 
 
-
 =head2 port()
 
 Used internally. Accesses the C<Device::SerialPort> underlying object. If you need to
@@ -1186,6 +1235,12 @@ experiment or do low-level serial calls, you may want to access this. Please rep
 any usage of this kind, because probably (?) it is possible to include it in a higher
 level method.
 
+As of 1.52, C<port()> will automatically try to reconnect if it detects
+a bogus underlying port object. It will reconnect with the same options used
+when C<connect()>ing the first time.
+
+If no connection has taken place yet, then B<no automatic reconnection>
+will be attempted.
 
 =head2 repeat()
 
@@ -1316,8 +1371,10 @@ Waits (yea) for a given amount of time (in milliseconds). Usage:
 Wait is implemented via C<select> system call.
 Don't know if this is really a problem on some platforms.
 
+=head2 write_drain()
 
-
+Only a simple wrapper around C<Device::SerialPort::write_drain> method.
+Disabled for Win32 platform, that doesn't have that.
 
 
 =head1 REQUIRES
@@ -1376,7 +1433,7 @@ Cosimo Streppone, L<cosimo@cpan.org>
 
 =head1 COPYRIGHT
 
-(C) 2002-2006 Cosimo Streppone, L<cosimo@cpan.org>
+(C) 2002-2010 Cosimo Streppone, L<cosimo@cpan.org>
 
 This library is free software; you can only redistribute it and/or
 modify it under the same terms as Perl itself.
